<?php
	require_once IWP_PLUGIN_PATH . 'includes/iwpPluginOptions.php';
	require_once IWP_PLUGIN_PATH . 'admin/includes/iwpAdminUtils.php';

	class iwpPublicWhatsAppChatController {
		/**
		 * Constructor
		 * No se usa
		 */
		public function __construct() {
			wp_register_style('indigitall-public-whatsAppChat-styles',
				IWP_PUBLIC_URL . 'views/whatsAppChat/css/iwp-public-whatsAppChat-styles.css?v=' . IWP_PLUGIN_VERSION);
			wp_register_script('indigitall-public-whatsAppChat-scripts',
				IWP_PUBLIC_URL . 'views/whatsAppChat/js/iwp-public-whatsAppChat-scripts.js?v=' . IWP_PLUGIN_VERSION);
			wp_register_script('indigitall-qr-code', IWP_PLUGIN_URL . 'includes/qr-code/qrcode-v2.js?v=' . IWP_PLUGIN_VERSION);

		}

		public function renderHtml() {
			$whatsAppChatStatus = filter_var(get_option(iwpPluginOptions::WH_STATUS, false), FILTER_VALIDATE_BOOLEAN);
			$whatsAppChatPhone = get_option(iwpPluginOptions::WH_PHONE, false);
			if ($whatsAppChatStatus && !empty($whatsAppChatPhone)) {
				wp_enqueue_style('indigitall-public-whatsAppChat-styles');
				wp_enqueue_script('indigitall-public-whatsAppChat-scripts');
				wp_enqueue_script('indigitall-qr-code');
				$whatsAppChatModel = $this->loadWhatsAppChatValues();

				$params = array(
					'whatsAppChatModel' => $whatsAppChatModel
				);
				$view = iwpAdminUtils::loadViewToVar(IWP_PUBLIC_PATH . 'views/whatsAppChat/iwpPublicWhatsAppChatView.php', $params);
				echo($view);
			}
			return false;
		}

		private function loadWhatsAppChatValues() {
			require_once IWP_PUBLIC_PATH . 'models/iwpWhatsAppChatModel.php';
			require_once IWP_PUBLIC_PATH . 'models/iwpWhatsAppChatIconModel.php';
			require_once IWP_PUBLIC_PATH . 'models/iwpWhatsAppChatWindowModel.php';
			require_once IWP_PUBLIC_PATH . 'models/iwpWhatsAppChatQrModel.php';

			$whStatus           = filter_var(get_option(iwpPluginOptions::WH_STATUS, false), FILTER_VALIDATE_BOOLEAN);
			$whPhone            = get_option(iwpPluginOptions::WH_PHONE,                              false);
			$whWelcome          = get_option(iwpPluginOptions::WH_CHAT_WELCOME_MESSAGE,               '');
			$whIconPosition     = get_option(iwpPluginOptions::WH_ICON_POSITION,                      '');
			$whIconColor        = get_option(iwpPluginOptions::WH_ICON_COLOR,                         iwpWhatsAppChatIconModel::DEFAULT_COLOR);
			$whIconImageId      = (int)get_option(iwpPluginOptions::WH_ICON_IMAGE_ID,                 false);
			$whIconImageName    = get_option(iwpPluginOptions::WH_ICON_IMAGE_NAME,                    '');
			$whIconTransparent  = filter_var(get_option(iwpPluginOptions::WH_ICON_TRANSPARENT_COLOR,  false), FILTER_VALIDATE_BOOLEAN);
			$whIconBubble       = get_option(iwpPluginOptions::WH_ICON_SPEECH_BALLOON,                false);
			$whIconBubbleText   = get_option(iwpPluginOptions::WH_ICON_SPEECH_BALLOON_TEXT,           '');
			$whIconSleep        = (int)get_option(iwpPluginOptions::WH_ICON_SLEEP,                    false);
			$whChatType         = get_option(iwpPluginOptions::WH_CHAT_TYPE,                          iwpWhatsAppChatModel::CHAT_TYPE_QR);
			$qrHeader           = get_option(iwpPluginOptions::WH_CHAT_QR_HEADER,                     '');
			$qrText             = get_option(iwpPluginOptions::WH_CHAT_QR_TEXT,                       '');
			$qrColor            = get_option(iwpPluginOptions::WH_CHAT_QR_COLOR,                      iwpWhatsAppChatQrModel::DEFAULT_COLOR);
			$whChatHeader       = get_option(iwpPluginOptions::WH_CHAT_HEADER,                        '');
			$whChatBody         = get_option(iwpPluginOptions::WH_CHAT_BODY,                          '');
			$whChatColor        = get_option(iwpPluginOptions::WH_CHAT_COLOR,                         iwpWhatsAppChatWindowModel::DEFAULT_COLOR);
			$whChatButtonText   = get_option(iwpPluginOptions::WH_CHAT_BUTTON_TEXT,                   '');
			$whChatButtonImage  = get_option(iwpPluginOptions::WH_CHAT_BUTTON_IMAGE_ID,               '');
			$whChatSleep        = (int)get_option(iwpPluginOptions::WH_CHAT_SLEEP,                    false);

			// Si el tipo de chat es QR y estamos en un dispositivo móvil, cambiar a "desactivado" para que se abra con el clic
			$whChatType = (($whChatType === iwpWhatsAppChatModel::CHAT_TYPE_QR) && IWP_IS_MOBILE) ? iwpWhatsAppChatModel::CHAT_TYPE_DISABLE : $whChatType;

			$icon = new iwpWhatsAppChatIconModel($whIconPosition, $whIconColor, $whIconImageId, $whIconImageName,
				$whIconTransparent, $whIconBubble, self::encodeDecodeText($whIconBubbleText), $whIconSleep);

			$window = new iwpWhatsAppChatWindowModel(self::encodeDecodeText($whChatHeader),
				self::encodeDecodeText($whChatBody), $whChatColor, self::encodeDecodeText($whChatButtonText),
				$whChatButtonImage, $whChatSleep);

			$qr = new iwpWhatsAppChatQrModel(self::encodeDecodeText($qrHeader), self::encodeDecodeText($qrText), $qrColor);

			$model = new iwpWhatsAppChatModel($whStatus, $whPhone, self::encodeDecodeText($whWelcome), $whChatType);
			$model->setIcon($icon);
			$model->setWindow($window);
			$model->setQr($qr);

			return $model;
		}

		private static function encodeDecodeText($text = '') {
			return self::decodeText(self::encodeText(stripslashes($text)));
		}

		private static function encodeText($text = '') {
			return htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
		}

		private static function decodeText($text = '') {
			return htmlspecialchars_decode($text, ENT_QUOTES);
		}
	}